-- ----------------------------------------------------------------------------------
--    (2 )
-- (.    PostgreSQL, . 39.9 "Trigger Procedures")
-- ----------------------------------------------------------------------------------

--    
DROP TABLE IF EXISTS progress;
CREATE TABLE progress
( mark_book numeric(5) NOT NULL,
  subject text NOT NULL,
  acad_year text NOT NULL,
  term numeric( 1 ) NOT NULL CHECK ( term = 1 OR term = 2 ),
  mark numeric( 1 ) NOT NULL CHECK ( mark >= 3 AND mark <= 5 ) DEFAULT 5,
  FOREIGN KEY ( mark_book )
    REFERENCES students ( mark_book )
    ON DELETE CASCADE
    ON UPDATE CASCADE
);

--    (     ,    ,
--    )
--      ,    
DROP TABLE IF EXISTS progress_2;
CREATE TABLE progress_2 
( mark_book numeric(5),
  subject text,
  acad_year text,
  term numeric( 1 ),
  mark numeric( 1 ),
  --   ()
  operation text,    --   (INSERT, UPDATE, DELETE)
  person text,       -- ,  
  moment timestamp   --  ,    
);
  
--   (   ,    )
CREATE OR REPLACE FUNCTION log_progress() RETURNS TRIGGER 
AS $log_progress_oper$
  BEGIN
    -- TG_OP --    PL/pgSQL,   
    --    
    IF TG_OP = 'INSERT' THEN
      -- NEW --  ,    students
      -- user -- ,    , 
      --         
      -- current_timestamp -- ,    
      INSERT INTO progress_2 VALUES ( NEW.*, TG_OP, user, current_timestamp );
      RETURN NEW;  --      NEW
    ELSIF TG_OP = 'UPDATE' THEN
      INSERT INTO progress_2 VALUES ( NEW.*, TG_OP, user, current_timestamp );
      RETURN NEW;  --      NEW
    ELSIF TG_OP = 'DELETE' THEN
      INSERT INTO progress_2 VALUES ( OLD.*, TG_OP, user, current_timestamp );
      RETURN OLD;  --      OLD
    END IF;
    RETURN NULL;   --  , ..   
                   --      students
                   -- (AFTER trigger)
  END;
$log_progress_oper$ LANGUAGE plpgsql;   --  PL/pgSQL 

--      log_progress()
--     ,    
-- (FOR EACH ROW)
CREATE TRIGGER log_progress_oper 
  AFTER INSERT OR UPDATE OR DELETE ON progress 
  FOR EACH ROW EXECUTE PROCEDURE log_progress();

--    ,       
--    (   ,    
--        )
CREATE OR REPLACE FUNCTION check_subject_count() RETURNS TRIGGER AS $check_subj_cnt$
  DECLARE
    exams_count integer;   --   ,  
  BEGIN
    SELECT count( subject ) INTO exams_count FROM progress
    WHERE mark_book = NEW.mark_book AND
          acad_year = NEW.acad_year AND
          term = NEW.term; 

    --          ...
    IF exams_count >= 3 THEN
      -- ...         INSERT
      --  UPDATE
      RAISE EXCEPTION '     ';
    END IF;

    --   ,        
    --  
    RETURN NEW;
  END;
$check_subj_cnt$ LANGUAGE plpgsql;

--      check_subject_count()
CREATE TRIGGER check_subj_cnt
  BEFORE INSERT OR UPDATE ON progress 
  FOR EACH ROW EXECUTE PROCEDURE check_subject_count();
